Extension { #name : 'JISCharacter' }

{ #category : 'Converting' }
JISCharacter class >> _twoByteEquivalent: aCharacter [

"Returns the JIS two-byte Character that is equivalent to the argument
 aCharacter.  If there is no two-byte equivalent, returns the JIS one-byte
 equivalent.  Two Characters are equivalent if they represent the same
 Character regardless of case or internal representation.  For example, a JIS
 one-byte a is equivalent to a one-byte or two-byte upper-case or lower-case a."

<primitive: 254>

aCharacter _validateClasses: { JISCharacter . Character }.
^ self _primitiveFailed: #_twoByteEquivalent: args: { aCharacter }

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> backspace [

"Returns the JIS back-space Character."

^ (JISCharacter withValue: 16#8)

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> cr [

"Returns the JIS carriage-return Character."

^ (JISCharacter withValue: 16#0D)

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> esc [

"Returns the JIS escape Character."

^ (JISCharacter withValue: 16#1B)

]

{ #category : 'Instance Creation' }
JISCharacter class >> fromStream: aStream [

"Returns the next JISCharacter in the stream aStream."

| result |
self _checkReadStream: aStream forClass: JapaneseString.
result:= aStream next.
result _validateClass: JISCharacter.
^result

]

{ #category : 'Instance Creation' }
JISCharacter class >> fromString: aJapaneseString [

"If aJapaneseString is a one-Character JapaneseString, returns the
 Character in aJapaneseString.  Otherwise, generates an error."

| result |
aJapaneseString _validateClass: JapaneseString.
(aJapaneseString size = 1)
ifTrue:
  [ result:= aJapaneseString at: 1.
    result _validateClass: JISCharacter.
    ^ result
  ]
ifFalse:
  [ self _errIncorrectFormat: aJapaneseString ]

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> lf [

"Returns the JIS line-feed Character."

^ (JISCharacter withValue: 16#0A)

]

{ #category : 'Storing and Loading' }
JISCharacter class >> loadFrom: passiveObj [

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

 | inst |
  inst := JISCharacter withValue:(Integer fromString:(passiveObj upToSeparator)).
  ^inst  "do not send hasRead: ,  result is special"
]

{ #category : 'Printable Characters' }
JISCharacter class >> lowercaseGreek [

"Returns an Array containing all lower-case Greek JISCharacters in alphabetic
 order."

| result |

result:= { } .
16#2641 to: 16#2658
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> lowercaseRussian [

"Returns an Array containing all lower-case Russian JISCharacters in alphabetic
 order."

| result |

result:= { } .
16#2751 to: 16#2771
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Instance Creation' }
JISCharacter class >> new [

"Disallowed.  You may not create new instances of JISCharacter."

self shouldNotImplement: #new

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> newPage [

"Returns the JIS new-page Character."

^ (JISCharacter withValue: 16#0C)

]

{ #category : 'Printable Characters' }
JISCharacter class >> oneByteDigits [

"Returns an Array containing one-byte JISCharacters representing the
 digits 0 through 9."

| result |

result:= { } .
16#30 to: 16#39
      do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> oneByteLowercaseRoman [

"Returns an Array containing all one-byte lower-case Roman JISCharacters
 in alphabetic order."

| result |

result:= { } .
1 to: 255 do: [:i | | c |
  c := self withValue: i.
  c isOneByteLowercaseRoman ifTrue: [ result add: c ]].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> oneByteUppercaseRoman [

"Returns an Array containing all one-byte upper-case Roman JISCharacters
 in alphabetic order."

| result |

result:= { } .
1 to: 255 do: [:i | | c |
  c := self withValue: i.
  c isOneByteUppercaseRoman ifTrue: [ result add: c ]].
^ result

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> space [

"Returns the JIS one-byte space Character."

^ (JISCharacter withValue: 16#20)

]

{ #category : 'Non-Printable Characters' }
JISCharacter class >> tab [

"Returns the JIS tab Character."

^ (JISCharacter withValue: 16#09)

]

{ #category : 'Printable Characters' }
JISCharacter class >> twoByteDigits [

"Returns an Array containing two-byte JISCharacters representing the
 digits 0 through 9."

| result |

result:= { } .
16#2330 to: 16#2339
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> twoByteLowercaseRoman [

"Returns an Array containing all two-byte lower-case Roman JISCharacters
 in alphabetic order."

| result |

result:= { } .
16#2361 to: 16#237A
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> twoByteUppercaseRoman [

"Returns an Array containing all two-byte upper-case Roman JISCharacters
 in alphabetic order."

| result |

result:= { } .
16#2341 to: 16#235A
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> uppercaseGreek [

"Returns an Array containing all upper-case Greek JISCharacters in
 alphabetic order."

| result |

result:= { } .
16#2621 to: 16#2638
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Printable Characters' }
JISCharacter class >> uppercaseRussian [

"Returns an Array containing all upper-case Russian JISCharacters in
 alphabetic order."

| result |

result:= { } .
16#2721 to: 16#2741
        do: [:i | result add: (JISCharacter withValue: i) ].
^ result

]

{ #category : 'Instance Creation' }
JISCharacter class >> withEUCValue: anEUCValue [

"Returns the JISCharacter with the specified EUC value."

<primitive: 82>

anEUCValue _validateClass: Integer.
^ anEUCValue _error: #rtErrBadEUCValue args:  #()

]

{ #category : 'Instance Creation' }
JISCharacter class >> withValue: aJISValue [

"Returns the JISCharacter with the specified JIS value.  Generates
 an error if aJISValue is greater than 65535 or less than 0."

<primitive: 252>

aJISValue _validateClass: Integer.
^ aJISValue _error: #numErrArgNotChr args: { JISCharacter }.

]

{ #category : 'Testing' }
JISCharacter >> _category [

" Returns the numeric category id for this character "

<primitive: 640>
self _primitiveFailed: #_category

]

{ #category : 'Comparing' }
JISCharacter >> < aCharacter [

"Returns true if the receiver is less than the argument.
 Returns false otherwise."

<primitive: 80>

^aCharacter > self

]

{ #category : 'Comparing' }
JISCharacter >> <= aCharacter [

"Returns true if the receiver is less than or equal to
 the argument.  Returns false otherwise."

^(self < aCharacter) or: [self = aCharacter]

]

{ #category : 'Comparing' }
JISCharacter >> = aCharacter [

"Returns true if the receiver is equal to the argument.  Returns false
 otherwise."

<primitive: 81>

(aCharacter isKindOf: AbstractCharacter)
ifTrue: [ ^ aCharacter = self ]
ifFalse: [ ^ false ]

]

{ #category : 'Comparing' }
JISCharacter >> > aCharacter [

"Returns true if the receiver is greater than the argument.
 Returns false otherwise."

^(self <= aCharacter) not

]

{ #category : 'Comparing' }
JISCharacter >> >= aCharacter [

"Returns true if the receiver is greater than or equal to
 the argument.  Returns false otherwise."

^(self < aCharacter) not

]

{ #category : 'Converting' }
JISCharacter >> asCharacter [

"Returns the (ASCII) Character corresponding to the receiver."

^ self asInteger asCharacter

]

{ #category : 'Converting' }
JISCharacter >> asciiValue [

"Returns the ASCII value (a SmallInteger) corresponding to the receiver."

^ self asCharacter codePoint

]

{ #category : 'Formatting' }
JISCharacter >> asEUCString [

"Returns an EUCString that represents the receiver."
| res |
(res := EUCString new) at: 1 put: self .
^ res

]

{ #category : 'Converting' }
JISCharacter >> asHankaku [

"Returns a JISCharacter that is the one-byte representation of a two-byte digit
 or two-byte Roman receiver.  If the receiver is not a two-byte digit or a
 two-byte Roman Character, returns the receiver."

<primitive: 248>

^ self _primitiveFailed: #asHankaku

]

{ #category : 'Converting' }
JISCharacter >> asInteger [

"Returns the JIS code of the receiver as a SmallInteger."

^ self jisValue

]

{ #category : 'Converting' }
JISCharacter >> asJapaneseString: aClass [

"Returns an instance of class aClass containing only the receiver.
 aClass must be a subclass of JapaneseString."

aClass validateSubclassOf: JapaneseString.
^ aClass withAll: { self }

]

{ #category : 'Converting' }
JISCharacter >> asJISCharacter [

"Returns the receiver."

^ self

]

{ #category : 'Converting' }
JISCharacter >> asLowercase [

"Returns a JISCharacter that is the lower-case Character corresponding to the
 receiver.  If the receiver is lower-case or has no case, this returns the
 receiver itself.  Returns a one-byte lower-case Roman Character if the
 receiver is a one-byte upper-case Roman Character.  Returns a two-byte
 lower-case Roman Character if the receiver is a two-byte upper-case Roman
 Character.  The only JISCharacters that have case distinction are Roman,
 Greek, and Russian."

<primitive: 249>

^ self _primitiveFailed: #asLowercase

]

{ #category : 'Formatting' }
JISCharacter >> asString [

"Returns a one-Character instance of String containing the receiver."
| res |
(res := String new:1) at: 1 put: self .
^ res

]

{ #category : 'Converting' }
JISCharacter >> asUppercase [

"Returns a JISCharacter that is the upper-case Character corresponding
 to the receiver.  If the receiver is upper-case or has no case, this
 returns the receiver itself.  Returns a one-byte upper-case Roman Character
 if the receiver is a one-byte lower-case Roman Character.  Returns a
 two-byte upper-case Roman Character if the receiver is a two-byte lower-case
 Roman Character.  The only JISCharacters that have case distinction are Roman,
 Greek, and Russian."

<primitive: 250>

^ self _primitiveFailed: #asUppercase

]

{ #category : 'Converting' }
JISCharacter >> asZenkaku [

"Returns a JISCharacter that is the two-byte representation of a one-byte digit
 or one-byte Roman receiver.  If the receiver is not a one-byte digit or
 one-byte Roman Character, returns the receiver."

<primitive: 251>

^ self _primitiveFailed: #asZenkaku

]

{ #category : 'Converting' }
JISCharacter >> codePoint [

"Returns the ASCII value (a SmallInteger) corresponding to the receiver."

^self asCharacter codePoint

]

{ #category : 'Storing and Loading' }
JISCharacter >> containsIdentity [

"Private."

^true

]

{ #category : 'Copying' }
JISCharacter >> copy [

"Returns the receiver.  (Does not create a new instance.)"

^ self

]

{ #category : 'Converting' }
JISCharacter >> digitValue [

"Returns a SmallInteger representing the value of the receiver,
 a digit, or returns nil if the receiver is not a digit."

(self isOneByteDigit)
  ifTrue: [ ^self jisValue - 16#30 ].

(self isTwoByteDigit)
  ifTrue: [ ^self jisValue - 16#2330]
  ifFalse: [ ^nil]

]

{ #category : 'Formatting' }
JISCharacter >> displayWidth [

"Returns the width necessary to display the receiver.  For a JISCharacter, this
 can be 1 or 2."

(self isOneByteCharacter)
  ifTrue: [ ^1 ]
  ifFalse: [ ^2 ]

]

{ #category : 'Accessing' }
JISCharacter >> eucValue [

"Returns the EUC code of the receiver as a SmallInteger."

<primitive: 253>

^ self _primitiveFailed: #eucValue

]

{ #category : 'Accessing' }
JISCharacter >> firstByte [

"Returns a SmallInteger representing the value of the leading byte of the JIS
 value of the receiver.  If the receiver is a one-byte JISCharacter, returns
 zero."

^ (self jisValue) bitShift: -8

]

{ #category : 'Testing' }
JISCharacter >> hasEUCFormat [

"Returns true if the receiver can be represented in EUC format.
 Returns false otherwise."

<primitive: 255>

^ self _primitiveFailed: #hasEUCFormat

]

{ #category : 'Testing' }
JISCharacter >> isDigit [

"Returns true if the receiver is a one- or two-byte digit.  Returns
 false otherwise."

^(self isOneByteDigit) or: [self isTwoByteDigit]

]

{ #category : 'Testing' }
JISCharacter >> isEquivalent: aCharacter [

"Returns true if the receiver is the same Character as the argument
 regardless of case or internal representation."

| argClass |
argClass:= aCharacter class.
((argClass == Character) or:
              [argClass == JISCharacter])
ifFalse: "argument is not a Character and is not a JISCharacter"
   [ ^aCharacter isEquivalent: self ].

^ (JISCharacter _twoByteEquivalent: self asUppercase) ==
  (JISCharacter _twoByteEquivalent: aCharacter asUppercase)

]

{ #category : 'Testing' }
JISCharacter >> isFirstLevelKanji [

"Returns true if the receiver is a first level Kanji Character.
 Returns false otherwise."

| byteOne byteTwo |

(self isOneByteCharacter)
ifTrue: [ ^ false].

byteOne:= self firstByte.
byteTwo:= self secondByte.

((byteOne >= 16#30) and: [byteOne <= 16#4E])
ifTrue:
   [ ^((byteTwo >= 16#21) and: [byteTwo <= 16#7E]) ]
ifFalse:
   [ ^(byteOne == 16#4F) and: [ (byteTwo >= 16#21) and: [byteTwo <= 16#53]]]

]

{ #category : 'Testing' }
JISCharacter >> isGreek [

"Returns true if the receiver is a Greek Character.  Returns false otherwise."

^ (self isLowercaseGreek) or: [self isUppercaseGreek]

]

{ #category : 'Testing' }
JISCharacter >> isHiragana [

"Returns true if the receiver is a Hiragana Character.  Returns false
 otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2473) and: [jisValue >= 16#2421]

]

{ #category : 'Testing' }
JISCharacter >> isJisAscii [

"Returns true if the receiver is a JIS-ASCII Character.  Returns false
 otherwise."

^(self jisValue) <= 127

]

{ #category : 'Testing' }
JISCharacter >> isKana [

"Returns true if the receiver is a Hiragana Character or a one- or
 two-byte Katakana Character.  Returns false otherwise."

^ (self isHiragana) or: [self isKatakana]

]

{ #category : 'Testing' }
JISCharacter >> isKanji [

"Returns true if the receiver is a Kanji Character.  Returns false otherwise."

^ (self isFirstLevelKanji) or: [self isSecondLevelKanji]

]

{ #category : 'Testing' }
JISCharacter >> isKatakana [

"Returns true if the receiver is a one- or two-byte Katakana Character.
 Returns false otherwise."

^ (self isOneByteKatakana) or: [self isTwoByteKatakana]

]

{ #category : 'Testing' }
JISCharacter >> isLineElement [

"Returns true if the receiver is a line element Character.  Returns
 false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2840) and: [jisValue >= 16#2821]

]

{ #category : 'Testing' }
JISCharacter >> isLowercase [

"Returns true if the receiver is a lower-case Character.  Returns false
 otherwise.  The only JISCharacters that have case distinction are
 Roman, Greek and Russian."

^ (self isOneByteLowercaseRoman) or:
     [ (self isTwoByteLowercaseRoman) or:
        [ (self isLowercaseGreek) or:
          [self isLowercaseRussian]]]

]

{ #category : 'Testing' }
JISCharacter >> isLowercaseGreek [

"Returns true if the receiver is a lower-case Greek Character.  Returns
 false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2658) and: [jisValue >= 16#2641]

]

{ #category : 'Testing' }
JISCharacter >> isLowercaseRussian [

"Returns true if the receiver is a lower-case Russian Character.  Returns
 false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2771) and: [jisValue >= 16#2751]

]

{ #category : 'Testing' }
JISCharacter >> isOneByteCharacter [

"Returns true if the receiver is a one-byte Character.  Returns false
 otherwise."

^ (self jisValue <= 16#FF)

]

{ #category : 'Testing' }
JISCharacter >> isOneByteDigit [

"Returns true if the receiver is a one-byte digit.  Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#39) and: [jisValue >= 16#30]

]

{ #category : 'Testing' }
JISCharacter >> isOneByteKatakana [

"Returns true if the receiver is a one-byte Katakana Character.
 Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
 ^ (jisValue <= 16#DF) and: [jisValue >= 16#A1]

]

{ #category : 'Testing' }
JISCharacter >> isOneByteLowercaseRoman [

"Returns true if the receiver is a one-byte lower-case Roman Character.
 Returns false otherwise."

^ self _category == 2

]

{ #category : 'Testing' }
JISCharacter >> isOneByteRoman [

"Returns true if the receiver is a one-byte Roman Character.
 Returns false otherwise."

^ (self isOneByteLowercaseRoman) or: [self isOneByteUppercaseRoman]

]

{ #category : 'Testing' }
JISCharacter >> isOneByteUppercaseRoman [

"Returns true if the receiver is a one-byte upper-case Roman Character.
 Returns false otherwise."

^ self _category == 1

]

{ #category : 'Testing' }
JISCharacter >> isRussian [

"Returns true if the receiver is a Russian Character.  Returns false
 otherwise."

^ (self isLowercaseRussian) or: [self isUppercaseRussian]

]

{ #category : 'Testing' }
JISCharacter >> isSecondLevelKanji [

"Returns true if the receiver is a second-level Kanji Character.
 Returns false otherwise."

| byteOne byteTwo |

(self isOneByteCharacter)
  ifTrue: [ ^ false].

byteOne:= self firstByte.
byteTwo:= self secondByte.

((byteOne >= 16#50) and: [byteOne <= 16#73])
  ifTrue:
    [ ^((byteTwo >= 16#21) and: [byteTwo <= 16#7E]) ]
  ifFalse:
    [ ^(byteOne == 16#74) and: [ (byteTwo >= 16#21) and: [byteTwo <= 16#24]]]

]

{ #category : 'Testing' }
JISCharacter >> isSpecial [

"Returns true if the receiver is a special object."

^ true

]

{ #category : 'Testing' }
JISCharacter >> isSpecialChar [

"Returns true if the receiver is a special Character.  Returns false
 otherwise."

| jisValue |

jisValue:= self jisValue.
((jisValue <= 47) and: [jisValue >= 32]) ifTrue: [ ^true ].
((jisValue <= 64) and: [jisValue >= 58]) ifTrue: [ ^true ].
((jisValue <= 96) and: [jisValue >= 91]) ifTrue: [ ^true ].
((jisValue <= 191) and: [jisValue >= 123]) ifTrue: [ ^true ].
(jisValue == 215) ifTrue: [ ^true ].
(jisValue == 247) ifTrue: [ ^true ].
((jisValue <= 16#217E) and: [jisValue >= 16#2121]) ifTrue: [ ^true ].
((jisValue <= 16#222E) and: [jisValue >= 16#2221]) ifTrue: [ ^true ].
((jisValue <= 16#2241) and: [jisValue >= 16#223A]) ifTrue: [ ^true ].
((jisValue <= 16#2250) and: [jisValue >= 16#224A]) ifTrue: [ ^true ].
((jisValue <= 16#226A) and: [jisValue >= 16#225C]) ifTrue: [ ^true ].
((jisValue <= 16#2279) and: [jisValue >= 16#2272]) ifTrue: [ ^true ].
^ (jisValue == 16#227E)

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteCharacter [

"Returns true if the receiver is a two-byte Character.  Returns false
 otherwise."

^ (self jisValue > 16#FF)

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteDigit [

"Returns true if the receiver is a two-byte digit.  Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2339) and: [jisValue >= 16#2330]

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteKatakana [

"Returns true if the receiver is a two-byte Katakana Character.
 Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2576) and: [jisValue >= 16#2521]

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteLowercaseRoman [

"Returns true if the receiver is a two-byte lower-case Roman Character.
 Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#237A) and: [jisValue >= 16#2361]

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteRoman [

"Returns true if the receiver is a two-byte Roman Character.
 Returns false otherwise."

^ (self isTwoByteLowercaseRoman) or: [self isTwoByteUppercaseRoman]

]

{ #category : 'Testing' }
JISCharacter >> isTwoByteUppercaseRoman [

"Returns true if the receiver is a two-byte upper-case Roman Character.
 Returns false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#235A) and: [jisValue >= 16#2341]

]

{ #category : 'Testing' }
JISCharacter >> isUppercase [

"Returns true if the receiver is an upper-case Character.  Returns false
 otherwise.  The only JISCharacters that have case distinction are
 Roman, Greek and Russian."

^ (self isOneByteUppercaseRoman) or:
     [ (self isTwoByteUppercaseRoman) or:
        [ (self isUppercaseGreek) or:
           [self isUppercaseRussian]]]

]

{ #category : 'Testing' }
JISCharacter >> isUppercaseGreek [

"Returns true if the receiver is an upper-case Greek Character.  Returns
 false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2638) and: [jisValue >= 16#2621]

]

{ #category : 'Testing' }
JISCharacter >> isUppercaseRussian [

"Returns true if the receiver is an upper-case Russian Character.  Returns
 false otherwise."

| jisValue |

jisValue:= self jisValue.
^ (jisValue <= 16#2741) and: [jisValue >= 16#2721]

]

{ #category : 'Accessing' }
JISCharacter >> jisValue [

"Returns the JIS code of the receiver as a SmallInteger."

<primitive: 247>

^ self _primitiveFailed: #jisValue

]

{ #category : 'Formatting' }
JISCharacter >> printOn: aStream [

"Puts a displayable representation of the receiver on the given stream."

aStream nextPutAll: ((self class name) asString , ' withValue: ' ,
                    self asInteger describe)

]

{ #category : 'Accessing' }
JISCharacter >> secondByte [

"Returns a SmallInteger representing the value of the last byte
 of the JIS value of the receiver."

^ (self jisValue) \\ 16#0100

]

{ #category : 'Storing and Loading' }
JISCharacter >> writeTo: passiveObj [

"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj writeClass: self class;
  nextPutAll: self jisValue asString;
  space

]
